#include "ESP32Console/Helpers/PWDHelpers.h"

#include <stdlib.h>
#include <stdio.h>
#include <Arduino.h>

namespace ESP32Console
{
    constexpr char *PWD_DEFAULT = (char*) "/";

    const char *console_getpwd()
    {
        char *pwd = getenv("PWD");
        if (pwd)
        { // If we have defined a PWD value, return it
            return pwd;
        }

        // Otherwise set a default one
        setenv("PWD", PWD_DEFAULT, 1);
        return PWD_DEFAULT;
    }

    const char *console_realpath(const char *path, char *resolvedPath)
    {
        String in = String(path);
        String pwd = String(console_getpwd());
        String result;
        // If path is not absolute we prepend our pwd
        if (!in.startsWith("/"))
        {
            result = pwd + "/" + in;
        }
        else
        {
            result = in;
        }
        
        realpath(result.c_str(), resolvedPath);
        return resolvedPath;
    }

    int console_chdir(const char *path)
    {
        char buffer[PATH_MAX + 2];
        console_realpath(path, buffer);

        size_t buffer_len = strlen(buffer);
        //If path does not end with slash, add it.
        if(buffer[buffer_len - 1] != '/')
        {
            buffer[buffer_len] = '/';
            buffer[buffer_len + 1] = '\0';
        }

        setenv("PWD", buffer, 1);

        return 0;
    }

}